-------------------------------------------------------------------------------
--
-- Title       : Filter_Memory
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description : Memory for pixels of image made of two FIFO buffers and 3x3 array
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

-------------------------------------------------------------------------------

entity Filter_Memory is
	generic (
		constant DEPTH : integer := 8;				-- size of the BRAM 
		constant V_DIM : integer := 256;			-- vertical dimension
		constant H_DIM : integer := 256				-- horizontal dimension
		);
	port(
		CLK 	 : in  std_logic;
		CLR 	 : in  std_logic;
		EN 	  	 : in  std_logic;
		PIXEL_IN : in  std_logic_vector(7 downto 0);
		P_11 	 : out std_logic_vector(7 downto 0);
		P_12 	 : out std_logic_vector(7 downto 0);
		P_13 	 : out std_logic_vector(7 downto 0);
		P_21 	 : out std_logic_vector(7 downto 0);
		P_22 	 : out std_logic_vector(7 downto 0);
		P_23 	 : out std_logic_vector(7 downto 0);
		P_31 	 : out std_logic_vector(7 downto 0);
		P_32 	 : out std_logic_vector(7 downto 0);
		P_33 	 : out std_logic_vector(7 downto 0) 
		);
end Filter_Memory;

-------------------------------------------------------------------------------

architecture Filter_Memory_Arch of Filter_Memory is	 
	
	-- constant for the FIFO length
	constant FIFO_LENGTH : integer := H_DIM-3;		
	
	-- registers for gathering pixels from line no. 3 
	signal pixel_11_reg :	std_logic_vector(7 downto 0);
	signal pixel_12_reg :	std_logic_vector(7 downto 0);
	signal pixel_13_reg :	std_logic_vector(7 downto 0);
	
	-- registers for gathering pixels from line no. 2
	signal pixel_21_reg :	std_logic_vector(7 downto 0);
	signal pixel_22_reg :	std_logic_vector(7 downto 0);
	signal pixel_23_reg :	std_logic_vector(7 downto 0);
	
	-- registers for gathering pixels from line no. 1
	signal pixel_31_reg :	std_logic_vector(7 downto 0);
	signal pixel_32_reg :	std_logic_vector(7 downto 0);
	signal pixel_33_reg :	std_logic_vector(7 downto 0);
	
	-- auxiliary signals for components
	signal dout1 : std_logic_vector(7 downto 0);
	signal dout2 : std_logic_vector(7 downto 0);		 
	
	-- declaration of FIFO buffer component
	component FIFO_Buffer is
		generic (
			constant DEPTH : integer;				-- size of the BRAM
			constant FIFO_LENGTH : integer			-- length of the FIFO
			);
		port (
			CLK  : in  std_logic;
			CLR  : in  std_logic;
			EN   : in  std_logic;
			DIN  : in  std_logic_vector(7 downto 0);
			DOUT : out std_logic_vector(7 downto 0)						
			);
	end component FIFO_Buffer;
	
	---------------------------------------------------------------------------
begin
	---------------------------------------------------------------------------
	
	-- FIFO buffer for first line
	U1: FIFO_Buffer
	generic map (
		DEPTH => DEPTH,
		FIFO_LENGTH => FIFO_LENGTH
		)
	port map (
		CLK => CLK,			 		
		CLR => CLR,
		EN => EN,
		DIN => pixel_31_reg,				
		DOUT => dout1				
		);
	
	-- FIFO buffer for second line
	U2: FIFO_Buffer
	generic map (
		DEPTH => DEPTH,
		FIFO_LENGTH => FIFO_LENGTH
		)
	port map (
		CLK => CLK,
		CLR => CLR,
		EN => EN,
		DIN => pixel_21_reg,				
		DOUT => dout2			
		);
	
	-- synchronization process
	Sync_Proc: process (CLK,EN) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then
				-- pixels assignment 
				pixel_33_reg <= PIXEL_IN;
				pixel_32_reg <= pixel_33_reg;
				pixel_31_reg <= pixel_32_reg;
				
				pixel_23_reg <= dout1;
				pixel_22_reg <= pixel_23_reg;
				pixel_21_reg <= pixel_22_reg;
				
				pixel_13_reg <= dout2;
				pixel_12_reg <= pixel_13_reg;
				pixel_11_reg <= pixel_12_reg;
			end if;
		end if;
	end process Sync_Proc;
	
	-- pixels assignment to the proper outputs including boundary conditions
	
	P_11 <= pixel_11_reg;
	
	P_12 <= pixel_12_reg; 
	
	P_13 <= pixel_13_reg;
	
	P_21 <= pixel_21_reg;	
	
	P_22 <= pixel_22_reg;	
	
	P_23 <= pixel_23_reg;	 
	
	P_31 <= pixel_31_reg;  
	
	P_32 <= pixel_32_reg;  
	
	P_33 <= pixel_33_reg;
	
	---------------------------------------------------------------------------
	
end Filter_Memory_Arch;
