-------------------------------------------------------------------------------
--
-- Title       : Get_Image
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description : This module gets samples from Matlab's array
--
-------------------------------------------------------------------------------

library IEEE; 	  
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

library aldec;
use aldec.matlab.all;

-------------------------------------------------------------------------------

entity Get_Image is
	generic (
		 V_IM_DIM : integer ;		-- maximum vertical size of image
		 H_IM_DIM : integer;		-- maximum horizontal size of image
		 D_IM_DIM : integer; 		-- number of parts of the image (3 for rgb)
		 NO_FRAMES : integer 		-- number of frames
		);
	port(
		CLK : in STD_LOGIC;
		CLR : in STD_LOGIC;
		DATA_OUT_EN : out STD_LOGIC;
		DATA_OUT : out STD_LOGIC_VECTOR(7 downto 0)
		);
end Get_Image;

-------------------------------------------------------------------------------

architecture Get_Image of Get_Image is
	
	-- internal signal for data sample
	signal sample_reg : std_logic_vector(7 downto 0);
	
	-- auxiliary signal for enable output port
	signal enable : std_logic := '0';
	
	---------------------------------------------------------------------------
begin
	---------------------------------------------------------------------------		
	-- This process get sample from arrey imported from Matlab
	--
	---------------------------------------------------------------------------		
	Get_Sample_Proc: process (CLK,CLR) is  
		
		-----------------------------------------------------------------------
		-- auxiliary variables
		variable run_once			: boolean := false;	  				-- auxiliary flag to init data reading
		variable im_array_id		: integer range 0 to 1;  			-- data sample array ID
		variable im_vdim_count		: integer range 1 to V_IM_DIM;  	-- vertical dimension counter
		variable im_hdim_count		: integer range 1 to H_IM_DIM;		-- horizontal dimension	counter
		variable im_ddim_count		: integer range 1 to D_IM_DIM;		-- depth dimension counter	 
		variable im_No_frames_count	: integer range 1 to NO_FRAMES+1;	-- number of frames	counter
		variable img_sample			: std_logic_vector(7 downto 0);		-- auxiliary variable to get sample
		
		------------------------------------------------------------------------		
	begin 
		------------------------------------------------------------------------ 		
		-- this part of the process should runs just once
		if(run_once = false) then
			run_once := true;
			ml_setup(false, 0, mxUINT8, mxUNSIGNED);
			-- run MATLAB M-file
			eval_string("get_img_sample");
			-- execute function tic
			eval_string("tic");
			-- get MATLAB object array 
			im_array_id := ml2hdl("img_src"); 
			-- in case of a trouble 
			assert im_array_id /= 0
			report (" Matlab GET_IMAGE read data error")
			severity FAILURE;
		end if;
		
		------------------------------------------------------------------------ 
		
		if CLR = '1' then
			enable <= '0'; 
			im_vdim_count := 1;
			im_hdim_count := 1;
			im_ddim_count := 1;	 
			im_No_frames_count := 1;
		elsif rising_edge(CLK) then
			-- reading data from arrey
			if(im_No_frames_count <= NO_FRAMES and im_array_id /= 0) then
				
				-- getting sample from MATLAB variable
				get_item( img_sample, mxUNSIGNED, im_array_id, (im_vdim_count,im_hdim_count,im_ddim_count,im_No_frames_count) ); 
				
				-- set dimensions
				if im_hdim_count = H_IM_DIM then
					im_hdim_count := 1;
					if im_vdim_count = V_IM_DIM then
						im_vdim_count := 1;
						if im_ddim_count = D_IM_DIM then
							im_ddim_count := 1;	
							im_No_frames_count := im_No_frames_count + 1;
						else
							im_ddim_count := im_ddim_count + 1;
						end if;
					else
						im_vdim_count := im_vdim_count + 1;
					end if;
				else
					im_hdim_count := im_hdim_count + 1;
				end if;
				
				enable <= '1';
				
				sample_reg <= img_sample; 
				
			end if;
		end if;		   
		
	end process Get_Sample_Proc;
	----------------------------------------------------------------------------
	
	DATA_OUT_EN	<= enable; 
	
	DATA_OUT <= sample_reg;
	
end Get_Image;
