-------------------------------------------------------------------------------
--
-- Title       : High_Boost_Filter
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description : Algorithm of mean removal filtering for 3x3 matrix
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;	
use IEEE.std_logic_arith.conv_std_logic_vector;
use IEEE.numeric_std.all;	

-------------------------------------------------------------------------------

entity High_Boost_Filter is 
	port(
		CLK 	  : in  std_logic;
		EN		  : in  std_logic;
		P_11 	  : in  std_logic_vector(7 downto 0);
		P_12 	  : in  std_logic_vector(7 downto 0);
		P_13 	  : in  std_logic_vector(7 downto 0);
		P_21 	  : in  std_logic_vector(7 downto 0);
		P_22 	  : in  std_logic_vector(7 downto 0);
		P_23 	  : in  std_logic_vector(7 downto 0);
		P_31 	  : in  std_logic_vector(7 downto 0);
		P_32 	  : in  std_logic_vector(7 downto 0);
		P_33 	  : in  std_logic_vector(7 downto 0);
		PIXEL_OUT : out std_logic_vector(7 downto 0)
		);
end High_Boost_Filter;

-------------------------------------------------------------------------------

architecture High_Boost_Filter of High_Boost_Filter is
	
	-- constant for dividing of result of highpass filtering
	constant div : integer := 0;
	
	-- auxiliary signals for substraction	  
	signal sub_reg_11 : signed(8 downto 0);   
	signal sub_reg_12 : signed(8 downto 0); 
	signal sub_reg_13 : signed(8 downto 0); 
	signal sub_reg_21 : signed(8 downto 0); 
	signal sub_reg_23 : signed(8 downto 0); 
	signal sub_reg_31 : signed(8 downto 0); 
	signal sub_reg_32 : signed(8 downto 0); 
	signal sub_reg_33 : signed(8 downto 0); 
	-- auxiliary signal for the ninth value of the center pixel
	signal aux_reg_01 : std_logic_vector(7 downto 0); 
	
	-- auxiliary signals for first addition
	signal sum_reg_11 : signed(9 downto 0); 	
	signal sum_reg_12 : signed(9 downto 0); 
	signal sum_reg_13 : signed(9 downto 0);    
	signal sum_reg_14 : signed(9 downto 0); 
	-- auxiliary signal for the last odd pixel in first addition
	signal aux_reg_11 : std_logic_vector(7 downto 0); 
	
	-- auxiliary signals for second addition
	signal sum_reg_21 : signed(10 downto 0); 	
	signal sum_reg_22 : signed(10 downto 0); 
	-- auxiliary signal for the last odd pixel in second addition
	signal aux_reg_21 : std_logic_vector(7 downto 0); 
	
	-- auxiliary signal for third addition
	signal sum_reg_31 : signed(11 downto 0); 
	-- auxiliary signal for the last odd pixel in third addition
	signal aux_reg_31 : std_logic_vector(7 downto 0); 
	
	-- auxiliary signal for the final sample
	signal final_sample : std_logic_vector(7 downto 0); 
	
	---------------------------------------------------------------------------
begin
	---------------------------------------------------------------------------
	Sub_Proc: process (CLK) is 
	begin
		if rising_edge(CLK) then
			if EN = '1' then
				sub_reg_11 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_11),9);  
				sub_reg_12 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_12),9);
				sub_reg_13 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_13),9);
				sub_reg_21 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_21),9);
				sub_reg_23 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_23),9);
				sub_reg_31 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_31),9);
				sub_reg_32 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_32),9);
				sub_reg_33 <= to_signed(conv_integer(P_22),9) - to_signed(conv_integer(P_33),9);
				aux_reg_01 <= P_22;
			end if;
		end if;
	end process Sub_Proc;
	
	---------------------------------------------------------------------------
	
	First_Add_Proc: process (CLK) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then 
				sum_reg_11 <= resize(sub_reg_11, 10) + resize(sub_reg_12, 10);	
				sum_reg_12 <= resize(sub_reg_13, 10) + resize(sub_reg_21, 10);
				sum_reg_13 <= resize(sub_reg_23, 10) + resize(sub_reg_31, 10);   
				sum_reg_14 <= resize(sub_reg_32, 10) + resize(sub_reg_33, 10);
				aux_reg_11 <= aux_reg_01;
			end if;
		end if;
	end process First_Add_Proc;
	
	---------------------------------------------------------------------------
	
	Second_Add_Proc: process (CLK) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then
				sum_reg_21 <= resize(sum_reg_11, 11) + resize(sum_reg_12, 11);	
				sum_reg_22 <= resize(sum_reg_13, 11) + resize(sum_reg_14, 11);
				aux_reg_21 <= aux_reg_11;
			end if;
		end if;
	end process Second_Add_Proc;
	
	---------------------------------------------------------------------------
	
	Third_Add_Proc: process (CLK) is
	begin
		if rising_edge(CLK) then
			if EN = '1' then
				sum_reg_31 <= resize(sum_reg_21, 12) + resize(sum_reg_22, 12);
				aux_reg_31 <= aux_reg_21;
			end if;
		end if;
	end process Third_Add_Proc;
	
	---------------------------------------------------------------------------
	
	Fourth_Add_and_Norm_Proc: process (CLK) is
		variable sum_temp : signed(12 downto 0);
	begin
		if rising_edge(CLK) then
			if EN = '1' then 
				sum_temp := to_signed(to_integer(sum_reg_31(11 downto div)),13) + to_signed(conv_integer(aux_reg_31),13);
				if sum_temp > 255 then
					final_sample <= x"FF";
				elsif sum_temp < 0 then
					final_sample <= x"00";
				else
					final_sample <= conv_std_logic_vector(to_integer(sum_temp(7 downto 0)),8);
				end if;
			end if;
		end if;
	end process Fourth_Add_and_Norm_Proc;
	
	---------------------------------------------------------------------------
	
	PIXEL_OUT <= final_sample;
	
end High_Boost_Filter;
